/*
 * Author: Konrad Rzeszutek <conradus@iname.com>
 * Date:   Tue, Sep 21 1999
 * File:   Editor2D/DrawFill.java
 *
 * Notes:
 *  
 *  This class fills polygons, circles with a nice white color (or with stripes).
 *  
 */

package Editor2D;

import java.awt.*;
import Editor2D.*;

public class DrawFill extends MyPrimitives {
    /*
     * Initializes the DrawFill object
     *
     * require:
     *   OffG != null  && canvas != null  && list != null
     *   ( (state == MyPrimitives.DRAW_NO_PATTERN) || (state == MyPrimitives.DRAW_PATTERN_ON)
     *   || (state == MyPrimitives.DRAW_PATTERN_OFF) || (state == MyPrimitives.DRAW_THICK_ON)
     *   || (state == MyPrimitives.DRAW_THICK_OFF))
     */
    public DrawFill (Graphics OffG, Canvas canvas, MyList list,int state) {
	
	super();
	super.myState = state;  
	super.maxPoints = 0;  // We use this, so that any MyPrimitives methods will return "nothingnes"
	this.canvas = canvas;
	this.list = list;
	this.OffG = OffG;
	
	pt = null;
	index = -1;           // Our special little counter used to point to the object we are fillig.
    }
    
    /*
     * Marks and computes the distance to the nearest DrawCircle or DrawPolygon and fills it.
     *
     * require:
     *   p instanceof Point
     *
     */
    public void add(Point pt) {
	
	
	this.pt = pt;
	this.draw();
    }
    
    /*
     * Fill the specified object. If object has been deleted or moved in the list
     * then nothing will be filled.
     *
     * note:
     *  if (isDeleted()) then this won't do a single thing.
     */
    public void draw () {
	
	int i;			   // Simple counter
	int currentD, myDistance;  // For distance calculations.
	
	
	if ((list.length() >= 0) && (pt != null) && (!deleted)) { // Do we have any other objects on the list?
	    // Is this the first time?
	    if (index < 0) {
		
		myDistance = currentD = list.itemAt(0).distance(pt);
		// Find the nearest object
		for (i=0;i<= list.length();i++) { // Travese thru the list of objects.
		    currentD = list.itemAt(i).distance(pt);
		    if ((currentD <= 500 && currentD <= myDistance)) { // Is it far enough?
			myDistance =  currentD;
			index = i;		  
		    }
		}
		
	    } 
	    // From here we should have a valid index. 
	    
	    if ( (index != -1) ) 
		if ( !list.itemAt(index).isDeleted()) {
		    
		    if (list.itemAt(index).isPolygon())   // yes, and its a polygon
			this.drawPolygon();
		    if (list.itemAt(index).isCircle())    // yes, and its a circle 
		    this.drawCircle();
		}	    
	}
    }
    
    /*
     * Finds the distance between points in this object
     * and the given point.
     *
     * require:
     *  pt instanceof Point
     *
     * return
     *  MyPrimitives.POS_INFINITY;
     *
     */
    public int distance (Point pt) {
	
	return MyPrimitives.POS_INFINITY;
    }
    
    /*
     * Changes the way the object is being drawn.
     *
     * require:
     *  (state == MyPrimitives.DRAW_NO_PATTERN) || (state == MyPrimitives.DRAW_PATTERN_ON)
     *   || (state == MyPrimitives.DRAW_PATTERN_OFF) || (state == MyPrimitives.DRAW_THICK_ON)
     *   || (state == MyPrimitives.DRAW_THICK_OFF))
     *
     */
    public void  setChanges(int state) {
	
	// Check to see if we already drew, if so, forget about this.
	
	if (index < 0)
	    super.setChanges(state);
    }
    /*
     * Returns a representation of the objects actions
     *
     * return:
     *  dump() instanceof String.
     */
    public String dump () {
	
	int i;	
	String temp = "\nOperation: fill ";
	
	
	if ( (index == -1))
	    temp+= "[TARGET PRIMITIVE NONEXISTENT]";
	else if (list.itemAt(index).isDeleted())
  	    temp+= "[TARGET PRIMITIVE DELETED]";
	if (deleted) 
	    temp+="[DELETED]";
	
	return temp;
	
    }

    /*
     * Private method which finds the edges of the polygon and fills the object.
     *
     * 
     */
    private void drawPolygon() {
	
	int i,ints;              // Simple counters
	int n = list.itemAt(index).getEdgeLength(); // Amount of edges/points the polygon has
	
	int ymin, ymax,y,xx;     
	int[] x;
	Point pt1;
	Point pt0;
	Color foreground = OffG.getColor();
	Color background = canvas.getBackground();
	
	if (n > 1) { // Better have more then one edge.
	    // Lets start with the first object.	
	    pt0 = list.itemAt(index).getEdge(0);
	    ymin = ymax = pt0.y;
	    for (i=0;i<n;i++) { // Go thru the edges and find ymin and ymax.
		pt0 = list.itemAt(index).getEdge(i);
		
	        if (pt0.y < ymin) 
		    ymin = pt0.y;
	        if (pt0.y > ymax)
		    ymax = pt0.y;
	    }
	    x = new int[n*n]; // is this enough? N is the amount of edges, so in worst
	    // case scenario the points are zigzaging like sine around y and would have n*n X's.
	    
	    for (y=ymin;y<=ymax;y++) {
		// find intersections of scan line y with all edges and leave them in 
		// array X. 
		ints = 0;
		
		for (i=0;i<n;i++) { // for all edges, and for all y's
		    pt0 = list.itemAt(index).getEdge(i); 
		    pt1 = list.itemAt(index).getEdge( (i+1) % n); 
		    if ( 
			(((y >= pt0.y) && (y <= pt1.y)) ||  // Is y between y0 and y1 
			 ((y <= pt0.y) && (y >= pt1.y))) &&
			((pt0.y != pt1.y)) &&		// Is y not horizontal
			((y != ymax)))			// Is y not the highest endpoint
			{
			    x[ints] = (int)( (double)(pt0.x) + // We have to convert first to double and then to int.
					     (((double)(y-pt0.y)) * ((double)(pt1.x - pt0.x)) / ((double)(pt1.y-pt0.y))));				
			    ints++;			   
			}
		}	    
		if (ints > 1) {
		    // Sort them.
		    QuickSort.IntegerQuickSort(x,0, ints-1);
		    for (i=0;i<ints;i++)
			for (xx=x[i];((xx<=x[i+1]) && ((i+1) < ints));xx++)  {
			    // Draw it.
			    if ((myState & DRAW_PATTERN_ON) == DRAW_PATTERN_ON)
				if (((xx+y) / 3) * 3 == (xx+y)) OffG.setColor(foreground);
				else OffG.setColor(background);
			    OffG.drawRect(xx,y,1,1);
			}  
		}
	    }
	    OffG.setColor(foreground); // In case we desynced on the last Y.
	}
    }
    
    /*
     * Private method which fills the circle.
     */
    private void drawCircle() {
	
	((DrawCircle)list.itemAt(index)).fill(myState); // Just calls the circle with fill's state.
	
    }
    
    private Point pt;      // The point nearest the choosen object
    private MyList list;   // List of all drawing objects.
    private Graphics OffG; // For drawing.
    private Canvas canvas; // For getting background color
    private int index =  -1; // The index on the 'list' of our choosen object.
    
} // End of DrawFill.java
