/*
 * Author:  Konrad Rzeszutek <conradus@iname.com> (only of the modifications)
 * Date:    Wed, Sep 22 1999
 * File:    Editor2D/Editor.java
 *
 * Disclaimer:
 *
 * This file has been shamelessly copied from Dr. Golden Richard III
 * AWT Lab1.java file. It is modified with his permission and all the
 * modifications where done by the author indicated above.
 *
 * Note:
 *
 *  This is a simple 2D Editor. It can draw lines, circle, and polygons.
 * It can also fill those objects and as well draw them thick and/or patterned
 * 
 *  This particular object handles the GUI and is responsible for propagating
 * state/change information to the "smart canvas" (MyCanvas.java) which 
 * in turn passes these information to appropiate drawing classes.
 *
 * Running this program is very easy:
 *
 *  %javac Editor2D.Editor
 *
 *  (make sure your CLASSPATH) includes the directory where Editor2D directory
 *   is located).
 *
 */
package Editor2D;

import java.awt.*;
import java.awt.event.*;
import java.io.*;


public class Editor extends Frame implements WindowListener, ActionListener,
    ItemListener, MouseListener,
    MouseMotionListener {
    
    // control buttons
    
    Button LineButton;
    Button PolyLineButton;
    Button CircleButton;
    Checkbox ThickCheckButton;
    Checkbox PatternCheckButton;
    Button FillButton;
    Button DeleteButton;
    Button DumpButton;
    Button ClearButton;
    Button ExitButton;
    
    
    MyCanvas canvas;      // "smart" canvas for drawing
    
    public Editor() {
	
	// constructor--initialize frame
	
	super("2D Editor");
	setup();
    }

    // initialize frame contents (buttons, etc.)

    public void setup() {

	Panel CanvasPanel;   // panel for drawing canvas
	Panel ButtonPanel;   // main button panel
	Panel ButtonPanel1;  // first row of buttons
	Panel ButtonPanel2;  // second row of buttons

	// size frame to 520x520 to accomodate a 500x375 drawing area + space for 
	// controls; must set size since no default size is provided


	setSize(520, 520);

	setLayout(new BorderLayout());

	// initialize buttons, make frame responsible for actions

	LineButton = new Button("Line");
	LineButton.addActionListener(this);
	PolyLineButton = new Button("PolyLine");
	PolyLineButton.addActionListener(this);
	CircleButton = new Button("Circle");
	CircleButton.addActionListener(this);
	ThickCheckButton = new Checkbox("Thick", null, false);
	ThickCheckButton.addItemListener(this);
	PatternCheckButton = new Checkbox("Pattern", null, false);
	PatternCheckButton.addItemListener(this);
	FillButton = new Button("Fill");
	FillButton.addActionListener(this);
	DeleteButton = new Button("Delete");
	DeleteButton.addActionListener(this);
	ClearButton = new Button("Clear");
	ClearButton.addActionListener(this);
	DumpButton = new Button("Dump");
	DumpButton.addActionListener(this);
	ExitButton = new Button("Exit");
	ExitButton.addActionListener(this);

	// initialize canvas 

	canvas = new MyCanvas(this);
	canvas.addMouseListener(this);
	canvas.addMouseMotionListener(this);
	canvas.setSize(500, 375);

	canvas.setBackground(Color.black);
	canvas.setForeground(Color.white);

	// add buttons to button panels

	ButtonPanel1 = new Panel();
	ButtonPanel1.setLayout(new FlowLayout(FlowLayout.CENTER));  // center buttons
	ButtonPanel1.add(LineButton);
	ButtonPanel1.add(PolyLineButton);
	ButtonPanel1.add(CircleButton);
	ButtonPanel1.add(ThickCheckButton);
	ButtonPanel1.add(PatternCheckButton);

	ButtonPanel2 = new Panel();
	ButtonPanel2.setLayout(new FlowLayout(FlowLayout.CENTER));  // center buttons
	ButtonPanel2.add(FillButton);
	ButtonPanel2.add(DeleteButton);
	ButtonPanel2.add(DumpButton);
	ButtonPanel2.add(ClearButton);
	ButtonPanel2.add(ExitButton);

	ButtonPanel = new Panel();
	ButtonPanel.setLayout(new GridLayout(2,1));   // 2 rows x 1 column
	ButtonPanel.add(ButtonPanel1);
	ButtonPanel.add(ButtonPanel2);
	

	// add canvas to canvas panel

	CanvasPanel = new Panel();
	CanvasPanel.setLayout(new BorderLayout());
	CanvasPanel.add(canvas,BorderLayout.CENTER);

	// put main panels into the frame

	add(ButtonPanel,BorderLayout.SOUTH);
	add(CanvasPanel,BorderLayout.CENTER);

	// frame is hidden until you show it

	setVisible(true);

	// care about window events
	
	addWindowListener(this);

    }

    // under the new event handling model, you have to "implement" all 
    // of these window operations if you're interested in any of them.
    // We only care about window close events.

    public void windowDeiconified(WindowEvent event) {
    }
    public void windowIconified(WindowEvent event) {
    }
    public void windowActivated(WindowEvent event) {
    }
    public void windowDeactivated(WindowEvent event) {
    }
    public void windowOpened(WindowEvent event) {
    }
    public void windowClosed(WindowEvent event) {
    }

    public void windowClosing(WindowEvent event) {

	// handle user closing frame

	setVisible(false); // hide frame first...
	dispose();         // release resources and destroy frame
	System.exit(0);    // exit program
	
    }

    public void mouseDragged(MouseEvent event) {
    }


    public void mousePressed(MouseEvent event) {

	// tell the canvas where the user clicked
	canvas.redraw(event.getPoint());
	
    }

    public void mouseReleased(MouseEvent event) {
    }
    public void mouseMoved(MouseEvent event) {
    }
    public void mouseEntered(MouseEvent event) {
    }
    public void mouseExited(MouseEvent event) {
    }
    public void mouseClicked(MouseEvent event) {
    }

    /*
     * Resets some of the visual aspects of the canvas
     */
    public void reset () {
	
	ThickCheckButton.setState(false);
	PatternCheckButton.setState(false);
    }
	
    public void itemStateChanged(ItemEvent event) {

	// handle check button events--first get source of event
	Object source = event.getSource();
	int state = event.getStateChange();

	if (source == PatternCheckButton) {
	    if (state == ItemEvent.SELECTED) {
		canvas.performSub(MyPrimitives.DRAW_PATTERN_ON);
	      // need to tell the canvas about this
	    }
	    else {
	      // need to tell the canvas about this
		canvas.performSub(MyPrimitives.DRAW_PATTERN_OFF);
	    }
	}
	else if (source == ThickCheckButton) {
	    if (state == ItemEvent.SELECTED) {
		canvas.performSub(MyPrimitives.DRAW_THICK_ON);
	      // need to tell the canvas about this
	    }
	    else {
		canvas.performSub(MyPrimitives.DRAW_THICK_OFF);

	      // need to tell the canvas about this
	    }
	}
    }


  public void actionPerformed(ActionEvent event) {
    
    // handle button events--first get source of event

    Object source = event.getSource();
    
    if (source == ExitButton) {
      setVisible(false); // hide frame first...
      dispose();         // release resources and destroy frame
      System.exit(0);    // exit program
    }
    else if (source == LineButton) {
      // need to tell canvas
	canvas.perform(MyCanvas.DRAW_LINE);
    }
    else if (source == PolyLineButton) {
      // need to tell canvas
	canvas.perform(MyCanvas.DRAW_POLYGONS);
    }
    else if (source == CircleButton) {
      // need to tell canvas
	canvas.perform(MyCanvas.DRAW_CIRCLE);
    }
    else if (source == FillButton) {
      // need to tell canvas
	canvas.perform(MyCanvas.DRAW_FILL);
    }
    else if (source == DeleteButton) {
      // need to tell canvas
	canvas.perform(MyCanvas.DRAW_DELETE);
    }
    else if (source == ClearButton) {
      // need to tell canvas
	canvas.perform(MyCanvas.DRAW_CLEAR);
    }
    else if (source == DumpButton) {
       	canvas.perform(MyCanvas.DRAW_DUMP);
      // need to tell canvas
    }
  }
    
    
  public void outputData(String str) {
      
      System.out.println(str);
  }
    
    public static void main(String args[]) {
	
	// startup 
    
	Editor lab = new Editor();
    }
    
}

	

	

	
	

    
